#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

out vec4 fragColor;

in VertexData {
	vec3 crntPos;
	vec3 normal;
	vec2 uv;
	vec3 color;
	vec4 fragPosDirLight;
} vert;

uniform sampler2D diffuseTexture;
uniform sampler2D dirLightShadowMap;

uniform vec3 cameraPos;
uniform vec3 dirLightPos;
uniform vec3 pointLightPos;

float calcShadow(vec3 normal, vec3 lightDirection) {
	float shadow = 0.0f;
	vec3 lightCoords = vert.fragPosDirLight.xyz / vert.fragPosDirLight.w; // Sets lightCoords to cull space
	if(lightCoords.z <= 1.0f) {
		lightCoords = (lightCoords + 1.0f) / 2.0f; // maps [-1, 1] to [0, 1]
		float currentDepth = lightCoords.z;

		//Apply PCF to prevent shadow acne
		float bias = max(0.025f * (1.0f - dot(normal, lightDirection)), 0.0005f);

		int sampleRadius = 1;
		vec2 pixelSize = 1.0 / textureSize(dirLightShadowMap, 0);
		for(int y = -sampleRadius; y <= sampleRadius; y++)
		{
		    for(int x = -sampleRadius; x <= sampleRadius; x++)
		    {
		        float pcfDepth = texture(dirLightShadowMap, lightCoords.xy + vec2(x, y) * pixelSize).r;
				shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;    
		    }    
		}
		shadow /= pow((sampleRadius * 2 + 1), 2);
	}
	return shadow;
}

vec3 calcDirectionalLight() {
	//material coefficients:
	float ambient = 0.1f;		//light with no origin (bounced off too many times)
	float diffuse = 0.4f;		//light that comes from one direction (brighter if it comes squarely down)
	float specular = 0.3f;		//shininess
	float specAlpha = 0.8f;		//amount of specular
	
	vec3 lightColor = vec3(0.8f);
	
	vec3 normal = normalize(vert.normal);
	vec3 lightDirection = normalize(dirLightPos - vert.crntPos);
	vec3 viewDirection = normalize(cameraPos - vert.crntPos);
	//--?--vec3 reflection = reflect(-lightDirection, normal);
	
	//calculate lighting
	diffuse = max(dot(normal, lightDirection), 0.0f) * diffuse;
	if (diffuse != 0.0f) {
		vec3 halfwayDir = normalize(lightDirection + viewDirection);  
		specular = pow(max(dot(normal, halfwayDir), 0.0), 16.0) * specAlpha;
	} else {
		specular = 0.0f;
	}
	
	//Shadow Calculation
	float shadow = calcShadow(normal, lightDirection);
	
	return lightColor * (ambient + (1.0 - shadow) * (diffuse + specular));
}

vec3 calcPointLight() {
	//material coefficients:
	float ambient = 0.1f;		//light with no origin (bounced off too many times)
	float diffuse = 0.8f;		//light that comes from one direction (brighter if it comes squarely down)
	float specular = 1.0f;		//shininess
	float constant =  1.0f;
	float linear = 0.09f;
	float quadratic = 0.032f;
	
	vec3 lightColor = vec3(1.0f);
	
	vec3 normal = normalize(vert.normal);
	vec3 lightDirection = normalize(pointLightPos - vert.crntPos);
	vec3 viewDirection = normalize(cameraPos - vert.crntPos);
	//--?--vec3 reflection = reflect(-lightDirection, normal);
	
    // attenuation (fading of point light)
    float distance = length(pointLightPos - vert.crntPos);
    float attenuation = 1.0 / (constant + linear * distance + quadratic * (distance * distance));
	
	//calculate lighting
	diffuse = max(dot(normal, lightDirection), 0.0f) * diffuse * attenuation;
	if (diffuse != 0.0f) {
		vec3 halfwayDir = normalize(lightDirection + viewDirection);  
		specular = pow(max(dot(normal, halfwayDir), 0.0), 32.0) * attenuation;
	} else {
		specular = 0.0f;
	}
	return lightColor * (ambient * attenuation + diffuse + specular);
}

void main() {
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	fragColor = vec4((calcDirectionalLight() + calcPointLight()) * texColor , 1);
}